SystemOrganization addCategory: #'Volunti-AnonymousAuthority-Model'!
SystemOrganization addCategory: #'Volunti-AnonymousAuthority-View'!

VOAuthority subclass: #VOAnonymousAuthority
	instanceVariableNames: 'users'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Volunti-AnonymousAuthority-Model'!

!VOAnonymousAuthority methodsFor: 'network' stamp: 'NicolasVanderAuwera 5/11/2012 12:25'!
authenticateOn: stream
	| user cert challenge response userInfo |
	user := stream receiveObject.
	userInfo := self userInfoWith: user.
	userInfo isNil
		ifTrue: [ 
			(cert := self requestCertificateFor: user asSubjectInfo) isNil
				ifTrue: [ 
					stream sendCommand: #error.
					^ self ].
			stream sendCommand: #success.
			challenge := self getChallenge.
			stream sendCommand: challenge greaseString.
			self users add: (VOUserInformation certificate: cert challenge: challenge subject: user asSubjectInfo) ]
		ifFalse: [ 
			stream sendCommand: #success.
			challenge := self getChallenge.
			stream sendCommand: challenge greaseString.
			userInfo challenge: challenge ]! !

!VOAnonymousAuthority methodsFor: 'network' stamp: 'NicolasVanderAuwera 5/10/2012 15:08'!
getChallenge
	^ SecureRandom new nextFrom: 1000 to: 9999! !

!VOAnonymousAuthority methodsFor: 'initialization' stamp: 'NicolasVanderAuwera 5/10/2012 15:04'!
initialize
	super initialize.
	users := OrderedCollection new! !

!VOAnonymousAuthority methodsFor: 'network' stamp: 'NicolasVanderAuwera 10/29/2012 16:33'!
requestCertificateFor: aSubjectInfo
	| client certificationAuth certificate |
	certificationAuth := self configurations at: VOCertificationAuthority key.
	client := SocketStream openConnectionToHostNamed: certificationAuth host port: certificationAuth port.
	client
		sendCommand: #certificateOf;
		sendObject: aSubjectInfo.
	certificate := client receiveObject.
	client close.
	^ certificate! !

!VOAnonymousAuthority methodsFor: 'network' stamp: 'NicolasVanderAuwera 5/11/2012 12:25'!
sendBlindedIdOn: stream with: sk
	| blind |
	blind := stream nextLine greaseInteger.
	stream sendCommand: (sk crypt: blind) greaseString! !

!VOAnonymousAuthority methodsFor: 'network' stamp: 'NicolasVanderAuwera 5/10/2012 15:49'!
userInfoWith: user
	^ self users detect: [ :info | info subjectInfo = user asSubjectInfo ] ifNone: [ nil ]! !

!VOAnonymousAuthority methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 5/10/2012 15:50'!
users
	^ users! !

!VOAnonymousAuthority methodsFor: 'network' stamp: 'NicolasVanderAuwera 5/11/2012 08:17'!
zeroKnowledgeOn: stream
	| response voterPk userInfo |
	userInfo := self userInfoWith: stream receiveObject.
	response := stream nextLine greaseInteger.
	voterPk := userInfo certificate tbsCertificate subjectPublicKeyInfo.
	userInfo challenge greaseString = (voterPk crypt: response greaseInteger) greaseString
		ifTrue: [ stream sendCommand: #success ]
		ifFalse: [ stream sendCommand: #error ]! !

VOManager subclass: #VOAnonymousManager
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Volunti-AnonymousAuthority-Model'!

!VOAnonymousManager class methodsFor: 'x509' stamp: 'NicolasVanderAuwera 12/11/2011 14:07'!
organisationUnit
	^ 'Anonymous Authority'! !

!VOAnonymousManager class methodsFor: 'network' stamp: 'NicolasVanderAuwera 12/1/2011 09:24'!
port
	^ 5634! !

!VOAnonymousManager methodsFor: 'network' stamp: 'NicolasVanderAuwera 7/27/2012 08:00'!
authenticateVoter
	self authorityOnStream authenticateOn: stream! !

!VOAnonymousManager methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 8/19/2012 09:02'!
authorityClass
	^ VOAnonymousAuthority! !

!VOAnonymousManager methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 7/23/2012 07:46'!
authorityFor: aProtocol
	^ VOAnonymousAuthority! !

!VOAnonymousManager methodsFor: 'network' stamp: 'NicolasVanderAuwera 5/10/2012 15:44'!
handle: aSymbol
	super handle: aSymbol.
	aSymbol = #authenticate
		ifTrue: [ self authenticateVoter ].
	aSymbol = #blindedid
		ifTrue: [ self sendBlindedId ].
	aSymbol = #response
		ifFalse: [ ^ self ].
	self responseCheck! !

!VOAnonymousManager methodsFor: 'network' stamp: 'NicolasVanderAuwera 7/27/2012 08:00'!
responseCheck
	self authorityOnStream zeroKnowledgeOn: stream! !

!VOAnonymousManager methodsFor: 'network' stamp: 'NicolasVanderAuwera 7/27/2012 08:00'!
sendBlindedId
	self authorityOnStream sendBlindedIdOn: stream with: self privateKey! !

VOAgentComponent subclass: #VOAnonymousAuthorityComponent
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Volunti-AnonymousAuthority-View'!

!VOAnonymousAuthorityComponent class methodsFor: 'class initialization' stamp: 'NicolasVanderAuwera 5/5/2012 07:27'!
initialize
	super initialize.
	self register! !

!VOAnonymousAuthorityComponent class methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 11/27/2011 13:24'!
url
	^ 'anonymousauth'! !

!VOAnonymousAuthorityComponent methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 7/23/2012 07:39'!
agentClass
	^ VOAnonymousManager! !

!VOAnonymousAuthorityComponent methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 11/27/2011 09:16'!
title
	^ 'Anonymous Authority'! !

Object subclass: #VOUserInformation
	instanceVariableNames: 'challenge certificate subjectInfo'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Volunti-AnonymousAuthority-Model'!

!VOUserInformation class methodsFor: 'instance creation' stamp: 'NicolasVanderAuwera 5/10/2012 15:35'!
certificate: aCert challenge: anInteger subject: aSubject
	^ (self new)
		certificate: aCert;
		challenge: anInteger;
		subjectInfo: aSubject;
		yourself! !

!VOUserInformation methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 5/10/2012 15:01'!
certificate
	^ certificate! !

!VOUserInformation methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 5/10/2012 15:01'!
certificate: anObject
	certificate := anObject! !

!VOUserInformation methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 5/10/2012 15:01'!
challenge
	^ challenge! !

!VOUserInformation methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 5/10/2012 15:01'!
challenge: anObject
	challenge := anObject! !

!VOUserInformation methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 5/10/2012 15:02'!
subjectInfo
	^ subjectInfo! !

!VOUserInformation methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 5/10/2012 15:02'!
subjectInfo: anObject
	subjectInfo := anObject! !
VOAnonymousAuthorityComponent initialize!
